/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include "memory.h"
#include "bio.h"

#define DEBUG(...)	fprintf(stderr, __VA_ARGS__)

// ***************************************************************************
// 
// ***************************************************************************
int b_write(int fd, void *b, size_t n)
{
	ssize_t r;
	size_t o;
	
	o = 0;
	
	while (o < n) {
		r = write(fd, b + o, n - o);
		
		if (r < 0) {
			if (errno != EINTR) {
				DEBUG("write(): %s\n", strerror(errno));
				return(-1);
			}
		} else if (r == 0) {
			DEBUG("write(): hangup\n");
			return(-1);
		} else {
			o += r;
		}
	}
	
	return(0);
}

// ***************************************************************************
// 
// ***************************************************************************
int b_read(int fd, void *b, size_t n)
{
	ssize_t r;
	size_t o;
	
	o = 0;
	
	while (o < n) {
		r = read(fd, b + o, n - o);
		
		if (r < 0) {
			if (errno != EINTR) {
				DEBUG("read(): %s\n", strerror(errno));
				return(-1);
			}
		} else if (r == 0) {
			DEBUG("read(): hangup\n");
			return(-1);
		} else {
			o += r;
		}
	}
	
	return(0);
}


// ***************************************************************************
// 
// ***************************************************************************
ssize_t b_read_eof(int fd, void *b, size_t n)
{
	ssize_t r;
	size_t o;
	
	o = 0;
	
	while (o < n) {
		r = read(fd, b + o, n - o);
		
		if (r < 0) {
			if (errno != EINTR) {
				DEBUG("read(): %s\n", strerror(errno));
				return(-1);
			}
		} else if (r == 0) {
			return(o);
		} else {
			o += r;
		}
	}
	
	return(o);
}

// ***************************************************************************
// 
// ***************************************************************************
size_t b_filesize(int fd)
{
	struct stat buf;
	
	if (fstat(fd, &buf)) {
		DEBUG("fstat(): %s\n", strerror(errno));
		return(0);
	}
	
	return(buf.st_size);
}

// ***************************************************************************
// 
// ***************************************************************************
int b_create_dir_path(char *path)
{
	struct stat st;
	size_t o, l;
	char *buf;
	
	if (!path)
		return(0);
	
	l = strlen(path);
	buf = xmalloc(l + 1);
	memcpy(buf, path, l + 1);
	
	o = 0;
	while (1) {
		while ((path[o] != 0) && (path[o] != '/'))
			o++;
		
		if (path[o] == 0)
			break;
		
		buf[o] = 0;
		
		// DEBUG("stat(%s)\n", buf);
		if ((o != 0) && (stat(buf, &st))) {
			// DEBUG("mkdir(%s)\n", buf);
			if (mkdir(buf, 0700)) {
				DEBUG("%s: %s\n", buf, strerror(errno));
				xfree(buf);
				return(-1);
			}
		}
		
		buf[o] = '/';
		
		o++;
	}
	
	xfree(buf);
	return(0);
}


