/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "bio.h"
#include "dateproc.h"

// ***************************************************************************
// 
// ***************************************************************************
#define DEBUG(...)	fprintf(stderr, __VA_ARGS__)

// ***************************************************************************
// 
// ***************************************************************************
time_t date(char *str)
{
	// fork/exec
	char *argv[5];
	int op[2];
	pid_t child;
	
	// data
	char buffer[256];
	ssize_t l;
	char *e;
	time_t t;
	
	if (pipe(op)) {
		DEBUG("pipe(): %s\n", strerror(errno));
		return(-1);
	}
	
	child = fork();
	
	if (child == -1) {
		DEBUG("fork(): %s\n", strerror(errno));
		close(op[0]);
		close(op[1]);
		return(-1);
	}
	
	if (child == 0) {
		close(op[0]);
		dup2(op[1], 1);

		if (op[1] > 1)
			close(op[1]);

		argv[0] = "/usr/bin/date";
		argv[1] = "+%s";
		argv[2] = "-d";
		argv[3] = str;
		argv[4] = NULL;
		
		execv(argv[0], argv);
		
		DEBUG("execve(): %s\n", strerror(errno));
		exit(1);
	}
	
	close(op[1]);

	l = b_read_eof(op[0], buffer, sizeof(buffer));
	if (l < 0)
		return(-1);
	
	close(op[0]);

	if (l == sizeof(buffer)) {
		buffer[sizeof(buffer) - 1] = 0;
		DEBUG("date(): date: output line too long (%s [...])\n",
			buffer);
		return(-1);
	}
	
	if (l < 1) {
		DEBUG("date(): date: empty output line\n");
		return(-1);
	}
	
	t = strtoll(buffer, &e, 0);
	
	if ((e == buffer) || (*e != '\n')) {
		DEBUG("date(): date: unable to parse output: %s\n",
			buffer);
		return(-1);
	}
	
	return(t);
}

