/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <time.h>
#include "daytime.h"


// ****************************************************************************
// 
// ****************************************************************************
time_t dt_now()
{
	return(time(NULL));
}


// ****************************************************************************
// 
// ****************************************************************************
/*
int dt_day(time_t t);
int dt_week(time_t t);
time_t dt_daytime(int day);
time_t dt_daytime_mid(int day);
time_t dt_weektime(int week);
time_t dt_weektime_mid(int week);

int dt_day(time_t t)
{
	return(t / (3600 * 24));
}

int dt_week(time_t t)
{
	// Week 0 starts on Sunday, Dec 28, 1969. Subtract 4 days.
	return((t + (3600 * 24 * 4)) / (3600 * 24 * 7));
}

time_t dt_daytime(int day)
{
	return(day * (3600 * 24));
}

time_t dt_daytime_mid(int day)
{
	return(day * (3600 * 24) + (3600 * 12));
}

time_t dt_weektime(int week)
{
	return(week * (3600 * 24 * 7));
}

time_t dt_weektime_mid(int week)
{
	return(week * (3600 * 24 * 7) + (3600 * 12));
}
*/

// ****************************************************************************
// 
// ****************************************************************************
/*
struct tm {
	int tm_sec;         // seconds 0..59 (60 for leap)
	int tm_min;         // minutes 0..59
	int tm_hour;        // hours 0..23
	int tm_mday;        // day of the month 1..31
	int tm_mon;         // month 0..11
	int tm_year;        // year (real year - 1900)
	int tm_wday;        // day of the week Sunday..Saturday - 0..6
	int tm_yday;        // day in the year 0..364 (or 365)
	int tm_isdst;       // daylight saving time
};
*/

static void dt_convert(dt_time *r, struct tm *c)
{
	r->sec = c->tm_sec;
	r->min = c->tm_min;
	r->hour = c->tm_hour;
	r->mday = c->tm_mday;
	r->mon = c->tm_mon + 1;
	r->year = c->tm_year + 1900;
	r->wday = c->tm_wday;
	r->yday = c->tm_yday;
	r->isdst = c->tm_isdst;
}

static void dt_convert_back(dt_time *r, struct tm *c)
{
	c->tm_sec = r->sec;
	c->tm_min = r->min;
	c->tm_hour = r->hour;
	c->tm_mday = r->mday;
	c->tm_mon = r->mon - 1;
	c->tm_year = r->year - 1900;
	c->tm_wday = r->wday;
	c->tm_yday = r->yday;
	c->tm_isdst = r->isdst;
}

// ****************************************************************************
// 
// ****************************************************************************
time_t dt_mktime(dt_time *r)
{
	struct tm c;
	time_t t;
	
	dt_convert_back(r, &c);
	t = mktime(&c);
	dt_convert(r, &c);
	return(t);
}

void dt_localtime(time_t t, dt_time *r)
{
	struct tm c;
	
	localtime_r(&t, &c);
	dt_convert(r, &c);
}

void dt_gmtime(time_t t, dt_time *r)
{
	struct tm c;
	
	gmtime_r(&t, &c);
	dt_convert(r, &c);
}

// ****************************************************************************
// 
// ****************************************************************************
int dt_sec(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.sec);
}

int dt_min(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.min);
}

int dt_hour(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.hour);
}

int dt_mday(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.mday);
}

int dt_mon(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.mon);
}

int dt_year(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.year);
}

int dt_wday(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	return(r.wday);
}

// ****************************************************************************
// 
// ****************************************************************************
time_t dt_daystart(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 0;
	
	return(dt_mktime(&r));
}

time_t dt_dayend(time_t t)
{
	return(dt_daystart(t + 24 * 3600));
}

// ****************************************************************************
// 
// ****************************************************************************
// We play it safe by:
// - computing the epoch of noon on the input day,
// - then subtracting the necessary number of days (in seconds),
// - then returning the epoch of noon on the resulting day.
// This is to handle the days that are not 24*3600 seconds long.
time_t dt_daymid(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 12;
	
	return(dt_mktime(&r));
}

time_t dt_monthstart(time_t t)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 12;
	r.mday = 1;
	
	return(dt_mktime(&r));
}

time_t dt_weekstart(time_t t, int first_wday)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 12;
	
	if (r.wday >= first_wday)
		t = dt_mktime(&r) - ((r.wday - first_wday) * 24 * 3600);
	else
		t = dt_mktime(&r) - ((7 + r.wday - first_wday) * 24 * 3600);
	
	return(dt_daymid(t));
}

// ****************************************************************************
// 
// ****************************************************************************
time_t dt_nextday(time_t t, int delta)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 12;
	
	if (delta >= 0) {
		return(dt_daymid(dt_mktime(&r) + (delta * 24 * 3600)));
	} else {
		delta = -delta;
		return(dt_daymid(dt_mktime(&r) - (delta * 24 * 3600)));
	}
}

time_t dt_nextweek(time_t t, int delta)
{
	return(dt_nextday(t, delta * 7));
}

time_t dt_nextmonth(time_t t, int delta)
{
	dt_time r;
	
	dt_localtime(t, &r);
	
	r.sec = r.min = 0;
	r.hour = 12;
	r.mday = 1;
	
	if (delta >= 0) {
		r.year += delta / 12;
		r.mon += delta % 12;
		if (r.mon > 12) {
			r.mon -= 12;
			r.year++;
		}
	} else {
		delta = -delta;
		r.year -= delta / 12;
		r.mon -= delta % 12;
		if (r.mon < 1) {
			r.mon += 12;
			r.year--;
		}
	}
	return(dt_mktime(&r));
}

time_t dt_nextyear(time_t t, int delta)
{
	return(dt_nextmonth(t, delta * 12));
}

