/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DAYTIME_H
#define DAYTIME_H
#include <time.h>

time_t dt_now();

typedef struct dt_time_t {
	int sec;         // seconds 0..59 (60 for leap)
	int min;         // minutes 0..59
	int hour;        // hours 0..23
	int mday;        // day of the month 1..31
	int mon;         // month 1..12
	int year;        // year
	int wday;        // day of the week Sunday..Saturday - 0..6
	int yday;        // day in the year 0..364 (or 365)
	int isdst;       // daylight saving time
} dt_time;

void dt_localtime(time_t t, dt_time *r);
void dt_gmtime(time_t t, dt_time *r);
time_t dt_mktime(dt_time *r);

int dt_sec(time_t t);
int dt_min(time_t t);
int dt_hour(time_t t);
int dt_mday(time_t t);
int dt_mon(time_t t);
int dt_year(time_t t);
int dt_wday(time_t t);

time_t dt_daystart(time_t t);
time_t dt_dayend(time_t t);

time_t dt_daymid(time_t t);
time_t dt_monthstart(time_t t);
time_t dt_weekstart(time_t t, int first_wday);

time_t dt_nextday(time_t t, int delta);
time_t dt_nextweek(time_t t, int delta);
time_t dt_nextmonth(time_t t, int delta);
time_t dt_nextyear(time_t t, int delta);

#endif

