/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CALENDAR_H
#define CALENDAR_H
#include "daytime.h"

// event
#define EVENT_SUMMARY		0
#define EVENT_LOCATION		1
#define EVENT_DESCRIPTION	2
#define EVENT_LINES		3

typedef struct event_t event;
typedef struct event_handle_t event_handle;
typedef struct event_list_t event_list;

struct event_t {
	time_t start, stop;
	char *data[EVENT_LINES];
};

event *event_new();
void event_set(event *ev, int line, char *content);
void event_copy(event *dst, event *src);
void event_destroy(event *ev);

// event handle and event list
struct event_handle_t {
	event *ev;
	event_handle *orig;
	event_handle *prev, *next;
};

struct event_list_t {
	int size;
	event_handle *head, *tail;
};

void evlist_init(event_list *l);
void evlist_clear(event_list *l);
event_handle *evlist_attach(event_list *l, event *e);
void evlist_detach(event_list *l, event_handle *h);
void evlist_select(event_list *dst, event_list *src, time_t begin, time_t end);
void evlist_slowsort(event_list *dst);
void evlist_destroy_all(event_list *l);

#endif

