/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef KEYS_H
#define KEYS_H
#include "config.h"
#include "terminal.h"

enum key_action_t {
	A_CAL_EXIT,
	A_CAL_ENTER,
	A_CAL_WEEK_PREV,
	A_CAL_WEEK_NEXT,
	A_CAL_SHIFT_M1,
	A_CAL_SHIFT_P1,
	A_CAL_SHIFT_M4,
	A_CAL_SHIFT_P4,
	A_CAL_DAY_PREV,
	A_CAL_DAY_NEXT,
	A_CAL_WEEK_START,
	A_CAL_WEEK_END,
	A_CAL_WEEK_START_M1,
	A_CAL_WEEK_START_P1,
	
	A_DAY_LEAVE,
	A_DAY_EV_PREV,
	A_DAY_EV_NEXT,
	A_DAY_DAY_PREV,
	A_DAY_DAY_NEXT,
	A_DAY_EV_FIRST,
	A_DAY_EV_LAST,
	A_DAY_SHIFT_M1,
	A_DAY_SHIFT_P1,
	A_DAY_SHIFT_M4,
	A_DAY_SHIFT_P4,
	A_DAY_EDIT,
	A_DAY_DELETE,
	
	A_SEL_CANCEL,
	A_SEL_CONFIRM,
	A_SEL_PREV,
	A_SEL_NEXT,
	A_SEL_M10,
	A_SEL_P10,
	A_SEL_FIRST,
	A_SEL_LAST,
	
	A_ED_SINGLE_EXIT,
	A_ED_SINGLE_CONFIRM,
	A_ED_SINGLE_PREV,
	A_ED_SINGLE_NEXT,
	A_ED_MULTI_EXIT,
	A_ED_MULTI_CONFIRM,
	A_ED_MULTI_PREV,
	A_ED_MULTI_NEXT,
	A_ED_MULTI_FIRST_PREV,
	A_ED_MULTI_UP,
	A_ED_MULTI_DOWN,
	A_ED_MULTI_UP_10,
	A_ED_MULTI_DOWN_10,
	A_ED_MORE,
	A_ED_LESS,
	A_ED_LEFT,
	A_ED_RIGHT,
	A_ED_HOME,
	A_ED_END,
	A_ED_DELETE,
	A_ED_BACKSPACE,
	A_ED_NEWLINE,
	A_ED_TAB,
	
	A_ACTIONS,
};

typedef struct key_str_t {
	size_t len;
	char str[TERM_KEY_STR_LENGTH];
} key_str;

#define KEY_ACTION_BINDINGS	32
typedef struct key_list_t {
	key_str action[A_ACTIONS][KEY_ACTION_BINDINGS];
} key_list;


void key_init(key_list *kl);
void key_clear(key_list *kl);

void key_config(key_list *kl, config *cfg);

int key_action(key_list *kl, term_keypress *k, int action);

int key_name(char *buff, size_t size, term_keypress *k);
int key_escape(char *buff, size_t size, term_keypress *k);

#endif

