/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "memory.h"
#include "log.h"

// **************************************************************************
// 
// **************************************************************************
void log_init(log *l)
{
	l->n = 0;
	l->lines = NULL;
}

void log_crop(log *l, int n)
{
	int i;
	
	if ((n > l->n) || (n < 0))
		return;
	
	for (i = n; i < l->n; i++)
		xfree(l->lines[i]);

	l->n = n;
}

void log_clear(log *l)
{
	log_crop(l, 0);
	tb_free(l->lines);
}


// **************************************************************************
// 
// **************************************************************************
void log_appendf(log *l, char *format, ...)
{
	va_list ap;
	char *r;
	
	va_start(ap, format);
	r = xvprintf(format, ap);
	va_end(ap);
	
	l->lines = tb_realloc(l->lines, (l->n + 1) * sizeof(char *));
	l->lines[l->n] = r;
	l->n++;
}

void log_prependf(log *l, char *format, ...)
{
	va_list ap;
	char *r;
	
	va_start(ap, format);
	r = xvprintf(format, ap);
	va_end(ap);
	
	l->lines = tb_realloc(l->lines, (l->n + 1) * sizeof(char *));
	memmove(&l->lines[1], &l->lines[0], l->n * sizeof(char *));
	l->lines[0] = r;
	l->n++;
}

void log_modif(log *l, int i, char *format, ...)
{
	va_list ap;
	char *r;
	
	if ((i < 0) || (i >= l->n))
		return;
	
	if (l->lines[i])
		xfree(l->lines[i]);
	
	va_start(ap, format);
	r = xvprintf(format, ap);
	va_end(ap);
	
	l->lines[i] = r;
}

// **************************************************************************
// 
// **************************************************************************

