/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef MEMORY_H
#define MEMORY_H
#include <stdlib.h>
#include <stdarg.h>

// failsafe funcs
void *dxmalloc(size_t n, const char *what,
	const char *func, const char *file, int line);
void *dxrealloc(void *p, size_t n, const char *pname, const char *what,
	const char *func, const char *file, int line);
void dxfree(void *p, const char *pname,
	const char *func, const char *file, int line);

#define xmalloc(n)	dxmalloc(n, #n, __func__, __FILE__, __LINE__)
#define xrealloc(p, n)	dxrealloc(p, n, #p, #n, __func__, __FILE__, __LINE__)
#define xfree(p)	dxfree(p, #p, __func__, __FILE__, __LINE__)

// tagged memory block
void *tb_master();
void *tb_sub_malloc(void *master, size_t n);
void *tb_malloc(size_t n);
void *tb_realloc(void *p, size_t n);
void tb_free(void *p);
size_t tb_size(void *p);
size_t tb_group_size(void *p);
size_t tb_space(void *p);
size_t tb_group_space(void *p);

char *xvprintf(const char *format, va_list ap);
char *xprintf(char *format, ...);

#endif

