/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef SCREEN_H
#define SCREEN_H
#include <stdio.h>
#include <stdint.h>
#include "terminal.h"
#include "utf8.h"

typedef uint64_t screen_flags;

typedef struct screen_t {
	int width, height;
	utf8_code *data;
	screen_flags *flags;
	utf8_code **drow;
	screen_flags **frow;
} screen;

#define SCREEN_WIDTH_MAX	1024

#define SCREEN_COLOR(fg, bg)	( (((screen_flags)fg) << 40) \
				| (((screen_flags)bg) << 16))

#define SCREEN_TRASH		0x100
#define SCREEN_BLANK		' '

void screen_init(screen *s);
void screen_clear(screen *s);

void screen_set(screen *s, screen_flags flags, utf8_code c);
void screen_resize(screen *s, int width, int height);

void screen_termflags(term *t, screen_flags f1);

void screen_fdump(screen *s, FILE *f);
void screen_tdump(screen *s, term *t);
void screen_tup(screen *s, term *t, screen *bkg);

void screen_multi(screen *s, int x, int y, int l,
	screen_flags flags, utf8_code c);
void screen_at(screen *s, int x, int y, int l, screen_flags flags, char *str);
void screen_atf(screen *s, int x, int y, int l,
	screen_flags flags, char *fmt, ...);

#endif

