/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "memory.h"
#include "table.h"


void table_init(table *a, int width, int height)
{
	int y;
	
	a->width = width;
	a->height = height;
	
	a->data = xmalloc(width * height * sizeof(cell));
	a->elem = xmalloc(height * sizeof(cell *));
	
	for (y = 0; y < height; y++) {
		a->elem[y] = &a->data[y * width];
	}
}

void table_clear(table *a)
{
	xfree(a->elem);
	xfree(a->data);
}

void table_fit(table *a, int sx, int sy, int sw, int sh)
{
	int wbl, hbl, wbonus, hbonus;
	int x, y, cx, cy, cw, ch;
	
	wbl = sw / a->width;
	hbl = sh / a->height;
	
	wbonus = sw % a->width;
	hbonus = sh % a->height;
	
	cy = 0;
	for (y = 0; y < a->height; y++) {
		ch = hbl + ((y < hbonus) ? 1 : 0);
		
		cx = 0;
		for (x = 0; x < a->width; x++) {
			cw = wbl + ((x < wbonus) ? 1 : 0);
			
			a->elem[y][x].x = sx + cx;
			a->elem[y][x].y = sy + cy;
			a->elem[y][x].width = cw;
			a->elem[y][x].height = ch;
			
			cx += cw;
		}
		
		cy += ch;
	}
}


