/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TERMINAL_H
#define TERMINAL_H
#include <stdlib.h>
#include "color.h"

// ******* ioctl / termios.h / man tty_ioctl
typedef struct term_t {
	int fd;
	int palette;
	unsigned long signaled;
	int width, height;
	struct {
		size_t n;
		char k[32];
	} buff;
} term;

int term_open(term *t);
void term_close(term *t);
int term_getsize(term *t);
int term_signaled(term *t);

// ******** control sequences / man console_codes
#define TERM_BOLD	1
#define TERM_DIM	2
#define TERM_UNDERLINE	4
#define TERM_BLINK	8
#define TERM_REVERSE	16
#define TERM_DEFAULTS	32
int term_altscreen(term *t, int alt);
int term_goto(term *t, int x, int y);
int term_font(term *t, int on, int off);
int term_color(term *t, color fg, color bg);
int term_write(term *t, void *buff, size_t sz);
int term_print(term *t, char *buff);

// ******** keypresses
// currently, utf8 requires 6, term_getkey requires 6
#define TERM_KEY_STR_LENGTH	8

typedef struct term_keypress_t {
	size_t n;
	char k[TERM_KEY_STR_LENGTH];
} term_keypress;

int term_getkey(term *t, term_keypress *k);
int term_match(term_keypress *k, char *s, size_t l);
int term_is(term_keypress *k, char *s);
int term_is_char(term_keypress *k, char c);
int term_is_ctrl(term_keypress *k, char c);

#define K_ESC		0
#define K_UP		1
#define K_DOWN		2
#define K_LEFT		3
#define K_RIGHT		4
#define K_INS		5
#define K_DEL		6
#define K_PG_UP		7
#define K_PG_DOWN	8
#define K_HOME		9
#define K_END		10
#define K_BACK		11
#define K_ENTER		12
#define K_TAB		13
#define K_SHIFT_TAB	14
int term_is_key(term_keypress *k, int key);

void term_key_dump(char *s, size_t l, int decimal);

#endif

