/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TEXTEDIT_H
#define TEXTEDIT_H
#include "terminal.h"
#include "keys.h"
#include "screen.h"
#include "utf8.h"

#define TED_MULTILINE	1
#define TED_MORELESS	2
#define TED_AT_END	4
typedef struct textedit_t {
	key_list *kl;
	screen *scr;
	
	int x, y, width, height;
	int flags, modified;
	
	int ox, oy, cx, cy;
	char *buff;
} textedit;

void ted_init(textedit *e, key_list *kl, screen *scr, int flags);
void ted_clear(textedit *e);
void ted_cursor_reset(textedit *e);
void ted_position(textedit *e, int x, int y, int w, int h);

char *ted_get(textedit *e);
void ted_set(textedit *e, char *s);
void ted_set_modified(textedit *e, int m);
int  ted_modified(textedit *e);

size_t ted_cursor_offset(textedit *e);
void ted_insert(textedit *e, size_t offs, char *s, size_t b);
void ted_delete(textedit *e, size_t offs);

#define ted_paint(e, fl, active)	ted_paint_internal((e), (fl))
void ted_paint_internal(textedit *e, screen_flags fl);
void ted_cursor_paint(textedit *e, term *tty, int active);

#define TED_CONTINUE	0
#define TED_CONFIRM	1
#define TED_CANCEL	2
#define TED_NEXT	3
#define TED_PREV	4
#define TED_MORE	5
#define TED_LESS	6
int ted_events(textedit *e, term_keypress *k);

#endif

