/*
    This file is part of "now", a simple calendar tool
    Copyright (C) 2012-2019  Laurent Poirrier

    "now" is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <string.h>
#include "config.h"
#include "screen.h"
#include "color.h"
#include "theme.h"

// **************************************************************************
// 
// **************************************************************************
#define THEME_DEFAULT	theme_dark

// **************************************************************************
// 
// **************************************************************************
static char *theme_base = "theme.base";

static char *theme_comp[T_COMPONENTS] =
{
"theme.ui.bkg",
"theme.ui.top-line",
"theme.ui.bottom-line",
"theme.ui.day-line",
"theme.ui.day-line-mark",
"theme.ui.day-line-sel",
"theme.ui.day-line-mark-sel",
"theme.ui.event",
"theme.ui.event-mark",
"theme.ui.event-sel",
"theme.ui.event-mark-sel",
"theme.ui.event-last-mod",
"theme.ev.bkg",
"theme.ev.editable",
"theme.sr.bkg",
"theme.sr.title",
"theme.sr.sel",
};

// **************************************************************************
// 
// **************************************************************************
static screen_flags theme_bw[T_COMPONENTS] =
{
// bkg, top, bottom
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_DIM,
TERM_DEFAULTS | TERM_DIM,
// day, sel
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
// ev, mark, sel, mark+sel, mod
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
0,
// ev bkg, ev edit
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS,
// sr bkg, sr title, sr sel
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE,
};

static screen_flags theme_bw_underline[T_COMPONENTS] =
{
// bkg, top, bottom
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_DIM | TERM_UNDERLINE,
TERM_DEFAULTS | TERM_DIM,
// day, sel
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
// ev, mark, sel, mark+sel, mod
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE,
TERM_DEFAULTS | TERM_REVERSE | TERM_BOLD,
TERM_UNDERLINE,
// ev bkg, ev edit
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS,
// sr bkg, sr title, sr sel
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_BOLD,
TERM_DEFAULTS | TERM_REVERSE,
};

static screen_flags theme_classic[T_COMPONENTS] =
{
// bkg, top, bottom
SCREEN_COLOR(0, 0xc0c0c0),
SCREEN_COLOR(0, 0xc0c0c0),
SCREEN_COLOR(0, 0xc0c0c0),
// day, sel
SCREEN_COLOR(0, 0xc0c0c0) | TERM_BOLD,
SCREEN_COLOR(0, 0xc0c0c0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x000080) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x000080) | TERM_BOLD,
// ev, mark, sel, mark+sel, mod
TERM_DEFAULTS,
TERM_DEFAULTS | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x000080) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x000080) | TERM_BOLD,
0,
// ev bkg, ev edit
SCREEN_COLOR(0, 0xc0c0c0) | TERM_BOLD,
TERM_DEFAULTS,
// sr bkg, sr title, sr sel
SCREEN_COLOR(0x808080, 0xc0c0c0),
SCREEN_COLOR(0xffffff, 0xc0c0c0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x000080) | TERM_BOLD,
};

static screen_flags theme_light[T_COMPONENTS] =
{
// bkg, top, bottom
SCREEN_COLOR(0x808080, 0xe0e0e0),
SCREEN_COLOR(0x404040, 0xe0e0e0),
SCREEN_COLOR(0x000000, 0xc0c0c0),
// day, sel
SCREEN_COLOR(0x000000, 0xc0c0c0),
SCREEN_COLOR(0x00bb00, 0xc0c0c0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
// ev, mark, sel, mark+sel, mod
SCREEN_COLOR(0x000000, 0xe0e0e0),
SCREEN_COLOR(0x008800, 0xe0e0e0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
SCREEN_COLOR(0x00ff00, 0x4090c0) | TERM_BOLD,
0,
// ev bkg, ev edit
SCREEN_COLOR(0x606060, 0xc0c0c0) | TERM_BOLD,
SCREEN_COLOR(0x000000, 0xe0e0e0),
// sr bkg, sr title, sr sel
SCREEN_COLOR(0x404040, 0xe0e0e0),
SCREEN_COLOR(0x000000, 0xe0e0e0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
};

static screen_flags theme_dark[T_COMPONENTS] =
{
// bkg, top, bottom
SCREEN_COLOR(0x808080, 0x202020),
SCREEN_COLOR(0xb0b0b0, 0x202020),
SCREEN_COLOR(0xffffff, 0x404040),
// day, sel
SCREEN_COLOR(0xffffff, 0x404040),
SCREEN_COLOR(0x00ff00, 0x404040) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
// ev, mark, sel, mark+sel, mod
SCREEN_COLOR(0xc0c0c0, 0x202020),
SCREEN_COLOR(0x00ff00, 0x202020) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
SCREEN_COLOR(0x00ff00, 0x4090c0) | TERM_BOLD,
0,
// ev bkg, ev edit
SCREEN_COLOR(0xc0c0c0, 0x202020) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x303030),
// sr bkg, sr title, sr sel
SCREEN_COLOR(0xb0b0b0, 0x202020),
SCREEN_COLOR(0xffffff, 0x202020) | TERM_BOLD,
SCREEN_COLOR(0xffffff, 0x4090c0) | TERM_BOLD,
};

// **************************************************************************
// 
// **************************************************************************
void theme_init(theme *th)
{
	memcpy(th->comp, THEME_DEFAULT, sizeof(th->comp));
}

void theme_clear(theme *th)
{
	(void)th;
}

// **************************************************************************
// 
// **************************************************************************
#define THEME_PARSE_MAX		256
static screen_flags theme_parse(char *s)
{
	char buff[THEME_PARSE_MAX];
	char *ss, *e;
	screen_flags f;
	color xg, fg, bg;
	size_t l, sl;
	int cc;
	
	// local copy
	l = strlen(s);
	if (l >= THEME_PARSE_MAX)
		l = THEME_PARSE_MAX - 1;
	memcpy(buff, s, l);
	buff[l] = 0;
	
	// chop
	f = 0;
	fg = bg = 0;
	cc = 0;
	
	while (l > 0) {
		chop(&s, &l, ',', &ss, &sl);
		
		if (sl == 0)
			continue;
		
		if (!strcmp(ss, "bold"))
			f |= TERM_BOLD;
		else if (!strcmp(ss, "dim"))
			f |= TERM_DIM;
		else if (!strcmp(ss, "underline"))
			f |= TERM_UNDERLINE;
		else if (!strcmp(ss, "blink"))
			f |= TERM_BLINK;
		else if (!strcmp(ss, "reverse"))
			f |= TERM_REVERSE;
		else if (!strcmp(ss, "defaults"))
			f |= TERM_DEFAULTS;
		else if (ss[0] == '#') {
			xg = color_parse(ss, &e);
			
			if (*e == 0) {
				if (cc == 0)
					fg = xg;
				else if (cc == 1)
					bg = xg;
				cc++;
			}
		}
	}
	
	if (cc == 1)
		f |= SCREEN_COLOR(fg, ~fg);
	else if (cc == 2)
		f |= SCREEN_COLOR(fg, bg);
	
	//printf(" #%06x, #%06x\n", fg, bg);
	return(f);
}

// **************************************************************************
// 
// **************************************************************************
void theme_config(theme *th, config *cfg)
{
	char *s;
	int c;
	
	s = config_get(cfg, theme_base, NULL);
	
	memcpy(th->comp, THEME_DEFAULT, sizeof(th->comp));

	if (s) {
		if (!strcmp(s, "bw"))
			memcpy(th->comp, theme_bw, sizeof(th->comp));
		if (!strcmp(s, "bw-underline"))
			memcpy(th->comp, theme_bw_underline, sizeof(th->comp));
		if (!strcmp(s, "classic"))
			memcpy(th->comp, theme_classic, sizeof(th->comp));
		if (!strcmp(s, "light"))
			memcpy(th->comp, theme_light, sizeof(th->comp));
		if (!strcmp(s, "dark"))
			memcpy(th->comp, theme_dark, sizeof(th->comp));
	}
	
	for (c = 0; c < T_COMPONENTS; c++) {
		s = config_get(cfg, theme_comp[c], NULL);
		
		if (s) {
			th->comp[c] = theme_parse(s);
		}
	}
}

// **************************************************************************
// 
// **************************************************************************

